@@ -29,10 +29,17 @@ _maix_image_module = Pybind11Extension(
             'ext_modules/libmaix/components/libmaix/lib/arch/r329/include/opencv4/'),
         get_incs(
             'ext_modules/libmaix/components/maix_cv_image/include'),
+        get_incs(
+            'ext_modules/libmaix/components/third_party/include'),
+        get_incs(
+            'ext_modules/libmaix/components/third_party/imlib/include'),
         get_incs(
             'ext_modules/_maix_image/include')
     ],
-    sources=get_srcs('ext_modules/_maix_image') + get_srcs('ext_modules/libmaix/components/maix_cv_image/src'),
+    sources=
+            get_srcs('ext_modules/_maix_image') +
+            get_srcs('ext_modules/libmaix/components/maix_cv_image/src') +
+            get_srcs('ext_modules/libmaix/components/third_party/imlib/src'),
     libraries=[
         "maix_image","maix_disp", "opencv_videoio", "opencv_highgui", "opencv_core", "opencv_imgproc", "opencv_imgcodecs", "opencv_freetype"
     ],
@@ -44,7 +51,7 @@ _maix_image_module = Pybind11Extension(
         "-Wl,-rpath=/usr/local/lib/python3.9/dist-packages/maix/_maix_opencv",
         "-Wl,-rpath=/usr/local/lib/python3.9/dist-packages/maix/",
     ],
-    extra_compile_args=['-std=c++11', '-std=gnu++11' ],
+    extra_compile_args=['-std=c++11', '-std=gnu++11', '-DIMLIB_CONFIG_H_FILE="costom_imlib_config.h"' ],
 )
 
 _maix_camera_module = Pybind11Extension(
@@ -63,14 +70,13 @@ _maix_camera_module = Pybind11Extension(
     extra_compile_args=['-std=c++11', '-std=gnu++11' ],
     extra_link_args=["-Wl,-rpath=/usr/local/lib/python3.9/dist-packages/maix"]
 )
-
 _maix_display_module = Pybind11Extension(
     name = "_maix_display",
     include_dirs=['ext_modules/_maix_display/include', 'ext_modules/libmaix/components/libmaix/include'],
     sources=get_srcs('ext_modules/_maix_display'),
     libraries=[
         "pthread",
-        # "maix_utils",
+        "maix_utils",
         "maix_disp",
         "maix_image",
     ],
@@ -85,12 +91,6 @@ _maix_display_module = Pybind11Extension(
 max_nn_srcs = get_srcs('ext_modules/_maix_nn/src')
 max_nn_srcs.extend(get_srcs('ext_modules/libmaix/components/libmaix/src'))
 max_nn_srcs.remove("ext_modules/libmaix/components/libmaix/src/libmaix.c")
-# max_nn_srcs.remove("ext_modules/_maix_nn/src/py_maix_nn_app.c")
-# max_nn_srcs.remove("ext_modules/_maix_nn/src/py_maix_nn_app_classifier.c")
-# max_nn_srcs.remove("ext_modules/_maix_nn/src/py_maix_nn_app_FaceRecognize.c")
-# max_nn_srcs.remove("ext_modules/_maix_nn/src/py_maix_nn_decoder_yolo2.c")
-# max_nn_srcs.remove("ext_modules/_maix_nn/src/py_maix_nn_decoder.c")
-# max_nn_srcs.remove("ext_modules/_maix_nn/src/py_maix_nn.c")
 _maix_nn_module = Extension('_maix_nn', include_dirs=['ext_modules/_maix_nn/include', 'ext_modules/libmaix/components/libmaix/include'],
                             sources=max_nn_srcs,
                             libraries=[
@@ -98,7 +98,7 @@ _maix_nn_module = Extension('_maix_nn', include_dirs=['ext_modules/_maix_nn/incl
 ],
     library_dirs=[ ext_so, ],
     # extra_link_args  = [ "-Wl,-z,origin", "-Wl,-rpath='$ORIGIN/maix'" ]
-    extra_compile_args=['-std=c++11', '-std=gnu++11' ],
+    extra_compile_args=['-std=c++11', '-std=gnu++11', '-DCONFIG_ARCH_R329'],
     extra_link_args=[
         "-Wl,-rpath=/usr/local/lib/python3.9/dist-packages/maix"
     ],
@@ -108,9 +108,9 @@ _maix_nn_module = Extension('_maix_nn', include_dirs=['ext_modules/_maix_nn/incl
 _maix_speech_module = Pybind11Extension("_maix_speech",
     include_dirs=[
         get_incs(
-            'ext_modules/_maix_speech/Maix-Speech/components/asr_lib/include'),
+            'ext_modules/libmaix/components/maix_speech/Maix-Speech/components/asr_lib/include'),
         get_incs(
-            'ext_modules/_maix_speech/Maix-Speech/components/utils/include')
+            'ext_modules/libmaix/components/maix_speech/Maix-Speech/components/utils/include')
     ],
     sources = get_srcs('ext_modules/_maix_speech', exclude=["utils", "projects"]),
     libraries=[
@@ -118,10 +118,10 @@ _maix_speech_module = Pybind11Extension("_maix_speech",
     ],
     library_dirs=[
         ext_so,
-        "./ext_modules/_maix_speech/Maix-Speech/components/asr_lib/lib/r329",
+        "./ext_modules/libmaix/components/maix_speech/Maix-Speech/components/asr_lib/lib/r329",
     ],
     extra_objects=[
-        "./ext_modules/_maix_speech/Maix-Speech/components/asr_lib/lib/r329/libms_asr_r329.a",
+        "./ext_modules/libmaix/components/maix_speech/Maix-Speech/components/asr_lib/lib/r329/libms_asr_r329.a",
     ],
     extra_compile_args=['-march=armv8-a', '-std=c++11', '-std=gnu++11'],
     # extra_compile_args=['-D__ARM__', '-D__ARMV7__', '-DCONF_KERNEL_IOMMU', '-DCONF_KERNEL_VERSION_4_9', '-std=c++11', '-std=gnu++11'],
@@ -131,6 +131,23 @@ _maix_speech_module = Pybind11Extension("_maix_speech",
     ],
 )
 
+_maix_nn_decode_retinaface_module =Pybind11Extension(
+    name = '_maix_nn_decoder_retinaface',
+    include_dirs=['ext_modules/libmaix/components/libmaix/include'],
+    sources=get_srcs("ext_modules/_maix_nn_decoder"),
+    libraries=[
+        "pthread",
+        # "maix_utils",
+        "maix_nn",
+        "maix_nn_decoder",
+    ],
+    library_dirs=[ ext_so, ],
+    # library_dirs=["/lib",  "/usr/lib", ext_so, ],
+    # extra_link_args  = [ "-Wl,-z,origin", "-Wl,-rpath='$ORIGIN/maix'" ]
+    extra_compile_args=['-std=c++11', '-std=gnu++11', '-DCONFIG_ARCH_R329'],
+    extra_link_args=["-Wl,-rpath=/usr/local/lib/python3.9/dist-packages/maix"]
+)
+
 
 _maix_modules = [
     libi2c_module,
@@ -140,7 +157,8 @@ _maix_modules = [
     _maix_image_module,
     _maix_camera_module,
     _maix_display_module,
-    _maix_nn_module
+    _maix_nn_module,
+    _maix_nn_decode_retinaface_module,
 ]
 
 _maix_data_files = [
@@ -151,7 +169,6 @@ _maix_data_files = [
 
 _maix_py_modules = [
     "numpy",
-    "Pillow",
     "rpyc",
     "gpiod",
     "evdev",
@@ -159,3 +176,5 @@ _maix_py_modules = [
     "pyserial",
     "zbarlight",
 ]
+
+
