@@ -358,18 +358,6 @@ maix_image &maix_image::_clear()
   memset(img_p_t, 0, this->_maix_image_size);
   return *this;
 }
-void maix_image::_load_freetype(std::string path)
-{
-  libmaix_cv_image_load_freetype(path.c_str());
-}
-
-py::tuple maix_image::_get_string_size(std::string str, double scale, int thickness)
-{
-  int w = 0, h = 0;
-  libmaix_cv_image_get_string_size(&w, &h, str.c_str(), scale, thickness);
-  return py::make_tuple(w, h);
-}
-
 void maix_image::_show()
 {
   if (NULL == this->_img)
@@ -431,7 +419,7 @@ int maix_image::_save(std::string file_path, std::string format)
 {
   if (NULL == this->_img)
     return -1;
-  libmaix_cv_image_draw_image_save(this->_img, file_path.c_str());
+  libmaix_cv_image_save_file(this->_img, file_path.c_str());
   return 0;
 }
 
@@ -514,14 +502,31 @@ maix_image &maix_image::_draw_line(int x1, int y1, int x2, int y2, std::vector<i
   return *this;
 }
 
-maix_image &maix_image::_draw_rectangle(int x, int y, int w, int h, std::vector<int> color, int thickness)
+maix_image &maix_image::_draw_cross(int x, int y, int c, int size, int thickness)
 {
   if (NULL == this->_img)
   {
     py::print("no img");
     return *this;
   }
-  libmaix_cv_image_draw_rectangle(this->_img, x, y, w, h, MaixColor(color[0], color[1], color[2]), thickness);
+  image_t img = {};
+  img.w = this->_img->width;
+  img.h = this->_img->height;
+  img.pixels = (uint8_t*)this->_img->data;
+  img.pixfmt = PIXFORMAT_RGB888;
+  imlib_draw_cross(&img, x, y, c, size, thickness);
+  return *this;
+}
+
+maix_image &maix_image::_draw_rectangle(int x1_x, int y1_y, int x2_w, int y2_h, std::vector<int> color, int thickness, int is_xywh)
+{
+  if (NULL == this->_img)
+  {
+    py::print("no img");
+    return *this;
+  }
+  if (is_xywh) x2_w = x1_x + x2_w, y2_h = y1_y + y2_h;
+  libmaix_cv_image_draw_rectangle(this->_img, x1_x, y1_y, x2_w, y2_h, MaixColor(color[0], color[1], color[2]), thickness);
   return *this;
 }
 
@@ -722,4 +727,123 @@ maix_image &maix_image::_set_pixel(int x, int y, std::vector<int> color)
   }
   libmaix_cv_image_set_pixel(this->_img, x, y, colot_tmp);
   return *this;
+}
+
+maix_image &maix_image::_hist_eq(bool adaptive,float clip_limit,maix_image & mask)
+{
+    if (NULL == this->_img)
+    {
+      py::print("no img");
+      return *this;
+    }
+
+    image_t img = {};
+    img.w = this->_img->width;
+    img.h = this->_img->height;
+    img.pixels = (uint8_t*)this->_img->data;
+    img.pixfmt = PIXFORMAT_RGB888;
+
+    image_t *mask_img = NULL;
+    if (NULL!=mask._img && this->_img->width == mask._img->width && this->_img->height == mask._img->height )
+    {
+      mask_img = (image_t *)malloc(sizeof(image_t));
+      if(mask_img)
+      {
+        mask_img->w = mask._img->width;
+        mask_img->h = mask._img->height;
+        mask_img->pixels = (uint8_t*)mask._img->data;
+        mask_img->pixfmt = PIXFORMAT_RGB888;
+      }
+    }
+
+    fb_alloc_mark();
+    if (adaptive)
+      imlib_clahe_histeq(&img, clip_limit, mask_img);
+    else
+      imlib_histeq(&img, mask_img);
+	  fb_alloc_free_till_mark();
+
+    if (NULL!=mask._img && this->_img->width == mask._img->width && this->_img->height == mask._img->height ) 
+      free(mask_img), mask_img = NULL;
+
+    return * this;
+}
+
+maix_image &maix_image::_gamma_corr(float gamma,float contrast,float brightness)
+{
+    if (NULL == this->_img)
+    {
+      py::print("no img");
+      return *this;
+    }
+    image_t img = {};
+    img.w = this->_img->width;
+    img.h = this->_img->height;
+    img.pixels = (uint8_t*)this->_img->data;
+    img.pixfmt = PIXFORMAT_RGB888;
+    fb_alloc_mark();
+    imlib_gamma_corr(&img,gamma,contrast,contrast);
+    fb_alloc_free_till_mark();
+    return * this;
+}
+
+maix_image &maix_image::_lens_corr(float strength, float zoom, float x_corr, float y_corr)
+{
+    if (NULL == this->_img)
+    {
+      py::print("no img");
+      return *this;
+    }
+
+    image_t img = {};
+    img.w = this->_img->width;
+    img.h = this->_img->height;
+    img.pixels = (uint8_t*)this->_img->data;
+    img.pixfmt = PIXFORMAT_RGB888;
+
+    fb_alloc_mark();
+    imlib_lens_corr(&img,strength,zoom,x_corr,y_corr);
+    fb_alloc_free_till_mark();
+    
+    return * this;
+}
+
+maix_image &maix_image::_mean(const int ksize, bool threshold, int offset,bool invert,maix_image & mask)
+{
+    if (NULL == this->_img)
+    {
+      py::print("no img");
+      return *this;
+    }
+
+    image_t img = {};
+    img.w = this->_img->width;
+    img.h = this->_img->height;
+    img.pixels = (uint8_t*)this->_img->data;
+    img.pixfmt = PIXFORMAT_RGB888;
+
+    image_t *mask_img = NULL;
+    if (NULL!=mask._img ) //check if mask is NULL
+    {
+      if (this->_img->width == mask._img->width && this->_img->height == mask._img->height ) //check if mask has same size with image
+      {
+      mask_img = (image_t *)malloc(sizeof(image_t));
+      if(mask_img)
+      {
+        mask_img->w = mask._img->width;
+        mask_img->h = mask._img->height;
+        mask_img->pixels = (uint8_t*)mask._img->data;
+        mask_img->pixfmt = PIXFORMAT_RGB888;
+      }
+      }
+      else printf("The size of mask is different with input image,use default mask!");
+    }
+
+    fb_alloc_mark();
+    imlib_mean_filter(&img,ksize,offset,threshold,invert,mask_img);
+    fb_alloc_free_till_mark();
+
+    if (mask_img!=NULL) free(mask_img), mask_img = NULL;
+
+    return * this;
 }
\ No newline at end of file
